""" Python Character Mapping Codec cp273 generated from 'python-mappings/CP273.TXT' with gencodec.py.

"""#"

import codecs

### Codec APIs

class Codec(codecs.Codec):

    def encode(self,input,errors='strict'):
        return codecs.charmap_encode(input,errors,encoding_table)

    def decode(self,input,errors='strict'):
        return codecs.charmap_decode(input,errors,decoding_table)

class IncrementalEncoder(codecs.IncrementalEncoder):
    def encode(self, input, final=False):
        return codecs.charmap_encode(input,self.errors,encoding_table)[0]

class IncrementalDecoder(codecs.IncrementalDecoder):
    def decode(self, input, final=False):
        return codecs.charmap_decode(input,self.errors,decoding_table)[0]

class StreamWriter(Codec,codecs.StreamWriter):
    pass

class StreamReader(Codec,codecs.StreamReader):
    pass

### encodings module API

def getregentry():
    return codecs.CodecInfo(
        name='cp273',
        encode=Codec().encode,
        decode=Codec().decode,
        incrementalencoder=IncrementalEncoder,
        incrementaldecoder=IncrementalDecoder,
        streamreader=StreamReader,
        streamwriter=StreamWriter,
    )


### Decoding Table

decoding_table = (
    '\x00'      #  0x00 -> NULL (NUL)
    '\x01'      #  0x01 -> START OF HEADING (SOH)
    '\x02'      #  0x02 -> START OF TEXT (STX)
    '\x03'      #  0x03 -> END OF TEXT (ETX)
    '\x9c'      #  0x04 -> STRING TERMINATOR (ST)
    '\t'        #  0x05 -> CHARACTER TABULATION (HT)
    '\x86'      #  0x06 -> START OF SELECTED AREA (SSA)
    '\x7f'      #  0x07 -> DELETE (DEL)
    '\x97'      #  0x08 -> END OF GUARDED AREA (EPA)
    '\x8d'      #  0x09 -> REVERSE LINE FEED (RI)
    '\x8e'      #  0x0A -> SINGLE-SHIFT TWO (SS2)
    '\x0b'      #  0x0B -> LINE TABULATION (VT)
    '\x0c'      #  0x0C -> FORM FEED (FF)
    '\r'        #  0x0D -> CARRIAGE RETURN (CR)
    '\x0e'      #  0x0E -> SHIFT OUT (SO)
    '\x0f'      #  0x0F -> SHIFT IN (SI)
    '\x10'      #  0x10 -> DATALINK ESCAPE (DLE)
    '\x11'      #  0x11 -> DEVICE CONTROL ONE (DC1)
    '\x12'      #  0x12 -> DEVICE CONTROL TWO (DC2)
    '\x13'      #  0x13 -> DEVICE CONTROL THREE (DC3)
    '\x9d'      #  0x14 -> OPERATING SYSTEM COMMAND (OSC)
    '\x85'      #  0x15 -> NEXT LINE (NEL)
    '\x08'      #  0x16 -> BACKSPACE (BS)
    '\x87'      #  0x17 -> END OF SELECTED AREA (ESA)
    '\x18'      #  0x18 -> CANCEL (CAN)
    '\x19'      #  0x19 -> END OF MEDIUM (EM)
    '\x92'      #  0x1A -> PRIVATE USE TWO (PU2)
    '\x8f'      #  0x1B -> SINGLE-SHIFT THREE (SS3)
    '\x1c'      #  0x1C -> FILE SEPARATOR (IS4)
    '\x1d'      #  0x1D -> GROUP SEPARATOR (IS3)
    '\x1e'      #  0x1E -> RECORD SEPARATOR (IS2)
    '\x1f'      #  0x1F -> UNIT SEPARATOR (IS1)
    '\x80'      #  0x20 -> PADDING CHARACTER (PAD)
    '\x81'      #  0x21 -> HIGH OCTET PRESET (HOP)
    '\x82'      #  0x22 -> BREAK PERMITTED HERE (BPH)
    '\x83'      #  0x23 -> NO BREAK HERE (NBH)
    '\x84'      #  0x24 -> INDEX (IND)
    '\n'        #  0x25 -> LINE FEED (LF)
    '\x17'      #  0x26 -> END OF TRANSMISSION BLOCK (ETB)
    '\x1b'      #  0x27 -> ESCAPE (ESC)
    '\x88'      #  0x28 -> CHARACTER TABULATION SET (HTS)
    '\x89'      #  0x29 -> CHARACTER TABULATION WITH JUSTIFICATION (HTJ)
    '\x8a'      #  0x2A -> LINE TABULATION SET (VTS)
    '\x8b'      #  0x2B -> PARTIAL LINE FORWARD (PLD)
    '\x8c'      #  0x2C -> PARTIAL LINE BACKWARD (PLU)
    '\x05'      #  0x2D -> ENQUIRY (ENQ)
    '\x06'      #  0x2E -> ACKNOWLEDGE (ACK)
    '\x07'      #  0x2F -> BELL (BEL)
    '\x90'      #  0x30 -> DEVICE CONTROL STRING (DCS)
    '\x91'      #  0x31 -> PRIVATE USE ONE (PU1)
    '\x16'      #  0x32 -> SYNCHRONOUS IDLE (SYN)
    '\x93'      #  0x33 -> SET TRANSMIT STATE (STS)
    '\x94'      #  0x34 -> CANCEL CHARACTER (CCH)
    '\x95'      #  0x35 -> MESSAGE WAITING (MW)
    '\x96'      #  0x36 -> START OF GUARDED AREA (SPA)
    '\x04'      #  0x37 -> END OF TRANSMISSION (EOT)
    '\x98'      #  0x38 -> START OF STRING (SOS)
    '\x99'      #  0x39 -> SINGLE GRAPHIC CHARACTER INTRODUCER (SGCI)
    '\x9a'      #  0x3A -> SINGLE CHARACTER INTRODUCER (SCI)
    '\x9b'      #  0x3B -> CONTROL SEQUENCE INTRODUCER (CSI)
    '\x14'      #  0x3C -> DEVICE CONTROL FOUR (DC4)
    '\x15'      #  0x3D -> NEGATIVE ACKNOWLEDGE (NAK)
    '\x9e'      #  0x3E -> PRIVACY MESSAGE (PM)
    '\x1a'      #  0x3F -> SUBSTITUTE (SUB)
    ' '         #  0x40 -> SPACE
    '\xa0'      #  0x41 -> NO-BREAK SPACE
    '\xe2'      #  0x42 -> LATIN SMALL LETTER A WITH CIRCUMFLEX
    '{'         #  0x43 -> LEFT CURLY BRACKET
    '\xe0'      #  0x44 -> LATIN SMALL LETTER A WITH GRAVE
    '\xe1'      #  0x45 -> LATIN SMALL LETTER A WITH ACUTE
    '\xe3'      #  0x46 -> LATIN SMALL LETTER A WITH TILDE
    '\xe5'      #  0x47 -> LATIN SMALL LETTER A WITH RING ABOVE
    '\xe7'      #  0x48 -> LATIN SMALL LETTER C WITH CEDILLA
    '\xf1'      #  0x49 -> LATIN SMALL LETTER N WITH TILDE
    '\xc4'      #  0x4A -> LATIN CAPITAL LETTER A WITH DIAERESIS
    '.'         #  0x4B -> FULL STOP
    '<'         #  0x4C -> LESS-THAN SIGN
    '('         #  0x4D -> LEFT PARENTHESIS
    '+'         #  0x4E -> PLUS SIGN
    '!'         #  0x4F -> EXCLAMATION MARK
    '&'         #  0x50 -> AMPERSAND
    '\xe9'      #  0x51 -> LATIN SMALL LETTER E WITH ACUTE
    '\xea'      #  0x52 -> LATIN SMALL LETTER E WITH CIRCUMFLEX
    '\xeb'      #  0x53 -> LATIN SMALL LETTER E WITH DIAERESIS
    '\xe8'      #  0x54 -> LATIN SMALL LETTER E WITH GRAVE
    '\xed'      #  0x55 -> LATIN SMALL LETTER I WITH ACUTE
    '\xee'      #  0x56 -> LATIN SMALL LETTER I WITH CIRCUMFLEX
    '\xef'      #  0x57 -> LATIN SMALL LETTER I WITH DIAERESIS
    '\xec'      #  0x58 -> LATIN SMALL LETTER I WITH GRAVE
    '~'         #  0x59 -> TILDE
    '\xdc'      #  0x5A -> LATIN CAPITAL LETTER U WITH DIAERESIS
    '$'         #  0x5B -> DOLLAR SIGN
    '*'         #  0x5C -> ASTERISK
    ')'         #  0x5D -> RIGHT PARENTHESIS
    ';'         #  0x5E -> SEMICOLON
    '^'         #  0x5F -> CIRCUMFLEX ACCENT
    '-'         #  0x60 -> HYPHEN-MINUS
    '/'         #  0x61 -> SOLIDUS
    '\xc2'      #  0x62 -> LATIN CAPITAL LETTER A WITH CIRCUMFLEX
    '['         #  0x63 -> LEFT SQUARE BRACKET
    '\xc0'      #  0x64 -> LATIN CAPITAL LETTER A WITH GRAVE
    '\xc1'      #  0x65 -> LATIN CAPITAL LETTER A WITH ACUTE
    '\xc3'      #  0x66 -> LATIN CAPITAL LETTER A WITH TILDE
    '\xc5'      #  0x67 -> LATIN CAPITAL LETTER A WITH RING ABOVE
    '\xc7'      #  0x68 -> LATIN CAPITAL LETTER C WITH CEDILLA
    '\xd1'      #  0x69 -> LATIN CAPITAL LETTER N WITH TILDE
    '\xf6'      #  0x6A -> LATIN SMALL LETTER O WITH DIAERESIS
    ','         #  0x6B -> COMMA
    '%'         #  0x6C -> PERCENT SIGN
    '_'         #  0x6D -> LOW LINE
    '>'         #  0x6E -> GREATER-THAN SIGN
    '?'         #  0x6F -> QUESTION MARK
    '\xf8'      #  0x70 -> LATIN SMALL LETTER O WITH STROKE
    '\xc9'      #  0x71 -> LATIN CAPITAL LETTER E WITH ACUTE
    '\xca'      #  0x72 -> LATIN CAPITAL LETTER E WITH CIRCUMFLEX
    '\xcb'      #  0x73 -> LATIN CAPITAL LETTER E WITH DIAERESIS
    '\xc8'      #  0x74 -> LATIN CAPITAL LETTER E WITH GRAVE
    '\xcd'      #  0x75 -> LATIN CAPITAL LETTER I WITH ACUTE
    '\xce'      #  0x76 -> LATIN CAPITAL LETTER I WITH CIRCUMFLEX
    '\xcf'      #  0x77 -> LATIN CAPITAL LETTER I WITH DIAERESIS
    '\xcc'      #  0x78 -> LATIN CAPITAL LETTER I WITH GRAVE
    '`'         #  0x79 -> GRAVE ACCENT
    ':'         #  0x7A -> COLON
    '#'         #  0x7B -> NUMBER SIGN
    '\xa7'      #  0x7C -> SECTION SIGN
    "'"         #  0x7D -> APOSTROPHE
    '='         #  0x7E -> EQUALS SIGN
    '"'         #  0x7F -> QUOTATION MARK
    '\xd8'      #  0x80 -> LATIN CAPITAL LETTER O WITH STROKE
    'a'         #  0x81 -> LATIN SMALL LETTER A
    'b'         #  0x82 -> LATIN SMALL LETTER B
    'c'         #  0x83 -> LATIN SMALL LETTER C
    'd'         #  0x84 -> LATIN SMALL LETTER D
    'e'         #  0x85 -> LATIN SMALL LETTER E
    'f'         #  0x86 -> LATIN SMALL LETTER F
    'g'         #  0x87 -> LATIN SMALL LETTER G
    'h'         #  0x88 -> LATIN SMALL LETTER H
    'i'         #  0x89 -> LATIN SMALL LETTER I
    '\xab'      #  0x8A -> LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
    '\xbb'      #  0x8B -> RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
    '\xf0'      #  0x8C -> LATIN SMALL LETTER ETH (Icelandic)
    '\xfd'      #  0x8D -> LATIN SMALL LETTER Y WITH ACUTE
    '\xfe'      #  0x8E -> LATIN SMALL LETTER THORN (Icelandic)
    '\xb1'      #  0x8F -> PLUS-MINUS SIGN
    '\xb0'      #  0x90 -> DEGREE SIGN
    'j'         #  0x91 -> LATIN SMALL LETTER J
    'k'         #  0x92 -> LATIN SMALL LETTER K
    'l'         #  0x93 -> LATIN SMALL LETTER L
    'm'         #  0x94 -> LATIN SMALL LETTER M
    'n'         #  0x95 -> LATIN SMALL LETTER N
    'o'         #  0x96 -> LATIN SMALL LETTER O
    'p'         #  0x97 -> LATIN SMALL LETTER P
    'q'         #  0x98 -> LATIN SMALL LETTER Q
    'r'         #  0x99 -> LATIN SMALL LETTER R
    '\xaa'      #  0x9A -> FEMININE ORDINAL INDICATOR
    '\xba'      #  0x9B -> MASCULINE ORDINAL INDICATOR
    '\xe6'      #  0x9C -> LATIN SMALL LETTER AE
    '\xb8'      #  0x9D -> CEDILLA
    '\xc6'      #  0x9E -> LATIN CAPITAL LETTER AE
    '\xa4'      #  0x9F -> CURRENCY SIGN
    '\xb5'      #  0xA0 -> MICRO SIGN
    '\xdf'      #  0xA1 -> LATIN SMALL LETTER SHARP S (German)
    's'         #  0xA2 -> LATIN SMALL LETTER S
    't'         #  0xA3 -> LATIN SMALL LETTER T
    'u'         #  0xA4 -> LATIN SMALL LETTER U
    'v'         #  0xA5 -> LATIN SMALL LETTER V
    'w'         #  0xA6 -> LATIN SMALL LETTER W
    'x'         #  0xA7 -> LATIN SMALL LETTER X
    'y'         #  0xA8 -> LATIN SMALL LETTER Y
    'z'         #  0xA9 -> LATIN SMALL LETTER Z
    '\xa1'      #  0xAA -> INVERTED EXCLAMATION MARK
    '\xbf'      #  0xAB -> INVERTED QUESTION MARK
    '\xd0'      #  0xAC -> LATIN CAPITAL LETTER ETH (Icelandic)
    '\xdd'      #  0xAD -> LATIN CAPITAL LETTER Y WITH ACUTE
    '\xde'      #  0xAE -> LATIN CAPITAL LETTER THORN (Icelandic)
    '\xae'      #  0xAF -> REGISTERED SIGN
    '\xa2'      #  0xB0 -> CENT SIGN
    '\xa3'      #  0xB1 -> POUND SIGN
    '\xa5'      #  0xB2 -> YEN SIGN
    '\xb7'      #  0xB3 -> MIDDLE DOT
    '\xa9'      #  0xB4 -> COPYRIGHT SIGN
    '@'         #  0xB5 -> COMMERCIAL AT
    '\xb6'      #  0xB6 -> PILCROW SIGN
    '\xbc'      #  0xB7 -> VULGAR FRACTION ONE QUARTER
    '\xbd'      #  0xB8 -> VULGAR FRACTION ONE HALF
    '\xbe'      #  0xB9 -> VULGAR FRACTION THREE QUARTERS
    '\xac'      #  0xBA -> NOT SIGN
    '|'         #  0xBB -> VERTICAL LINE
    '\u203e'    #  0xBC -> OVERLINE
    '\xa8'      #  0xBD -> DIAERESIS
    '\xb4'      #  0xBE -> ACUTE ACCENT
    '\xd7'      #  0xBF -> MULTIPLICATION SIGN
    '\xe4'      #  0xC0 -> LATIN SMALL LETTER A WITH DIAERESIS
    'A'         #  0xC1 -> LATIN CAPITAL LETTER A
    'B'         #  0xC2 -> LATIN CAPITAL LETTER B
    'C'         #  0xC3 -> LATIN CAPITAL LETTER C
    'D'         #  0xC4 -> LATIN CAPITAL LETTER D
    'E'         #  0xC5 -> LATIN CAPITAL LETTER E
    'F'         #  0xC6 -> LATIN CAPITAL LETTER F
    'G'         #  0xC7 -> LATIN CAPITAL LETTER G
    'H'         #  0xC8 -> LATIN CAPITAL LETTER H
    'I'         #  0xC9 -> LATIN CAPITAL LETTER I
    '\xad'      #  0xCA -> SOFT HYPHEN
    '\xf4'      #  0xCB -> LATIN SMALL LETTER O WITH CIRCUMFLEX
    '\xa6'      #  0xCC -> BROKEN BAR
    '\xf2'      #  0xCD -> LATIN SMALL LETTER O WITH GRAVE
    '\xf3'      #  0xCE -> LATIN SMALL LETTER O WITH ACUTE
    '\xf5'      #  0xCF -> LATIN SMALL LETTER O WITH TILDE
    '\xfc'      #  0xD0 -> LATIN SMALL LETTER U WITH DIAERESIS
    'J'         #  0xD1 -> LATIN CAPITAL LETTER J
    'K'         #  0xD2 -> LATIN CAPITAL LETTER K
    'L'         #  0xD3 -> LATIN CAPITAL LETTER L
    'M'         #  0xD4 -> LATIN CAPITAL LETTER M
    'N'         #  0xD5 -> LATIN CAPITAL LETTER N
    'O'         #  0xD6 -> LATIN CAPITAL LETTER O
    'P'         #  0xD7 -> LATIN CAPITAL LETTER P
    'Q'         #  0xD8 -> LATIN CAPITAL LETTER Q
    'R'         #  0xD9 -> LATIN CAPITAL LETTER R
    '\xb9'      #  0xDA -> SUPERSCRIPT ONE
    '\xfb'      #  0xDB -> LATIN SMALL LETTER U WITH CIRCUMFLEX
    '}'         #  0xDC -> RIGHT CURLY BRACKET
    '\xf9'      #  0xDD -> LATIN SMALL LETTER U WITH GRAVE
    '\xfa'      #  0xDE -> LATIN SMALL LETTER U WITH ACUTE
    '\xff'      #  0xDF -> LATIN SMALL LETTER Y WITH DIAERESIS
    '\xd6'      #  0xE0 -> LATIN CAPITAL LETTER O WITH DIAERESIS
    '\xf7'      #  0xE1 -> DIVISION SIGN
    'S'         #  0xE2 -> LATIN CAPITAL LETTER S
    'T'         #  0xE3 -> LATIN CAPITAL LETTER T
    'U'         #  0xE4 -> LATIN CAPITAL LETTER U
    'V'         #  0xE5 -> LATIN CAPITAL LETTER V
    'W'         #  0xE6 -> LATIN CAPITAL LETTER W
    'X'         #  0xE7 -> LATIN CAPITAL LETTER X
    'Y'         #  0xE8 -> LATIN CAPITAL LETTER Y
    'Z'         #  0xE9 -> LATIN CAPITAL LETTER Z
    '\xb2'      #  0xEA -> SUPERSCRIPT TWO
    '\xd4'      #  0xEB -> LATIN CAPITAL LETTER O WITH CIRCUMFLEX
    '\\'        #  0xEC -> REVERSE SOLIDUS
    '\xd2'      #  0xED -> LATIN CAPITAL LETTER O WITH GRAVE
    '\xd3'      #  0xEE -> LATIN CAPITAL LETTER O WITH ACUTE
    '\xd5'      #  0xEF -> LATIN CAPITAL LETTER O WITH TILDE
    '0'         #  0xF0 -> DIGIT ZERO
    '1'         #  0xF1 -> DIGIT ONE
    '2'         #  0xF2 -> DIGIT TWO
    '3'         #  0xF3 -> DIGIT THREE
    '4'         #  0xF4 -> DIGIT FOUR
    '5'         #  0xF5 -> DIGIT FIVE
    '6'         #  0xF6 -> DIGIT SIX
    '7'         #  0xF7 -> DIGIT SEVEN
    '8'         #  0xF8 -> DIGIT EIGHT
    '9'         #  0xF9 -> DIGIT NINE
    '\xb3'      #  0xFA -> SUPERSCRIPT THREE
    '\xdb'      #  0xFB -> LATIN CAPITAL LETTER U WITH CIRCUMFLEX
    ']'         #  0xFC -> RIGHT SQUARE BRACKET
    '\xd9'      #  0xFD -> LATIN CAPITAL LETTER U WITH GRAVE
    '\xda'      #  0xFE -> LATIN CAPITAL LETTER U WITH ACUTE
    '\x9f'      #  0xFF -> APPLICATION PROGRAM COMMAND (APC)
)

### Encoding table
encoding_table=codecs.charmap_build(decoding_table)



#  Author:      Fred L. Drake, Jr.
#               fdrake@acm.org
#
#  This is a simple little module I wrote to make life easier.  I didn't
#  see anything quite like it in the library, though I may have overlooked
#  something.  I wrote this when I was trying to read some heavily nested
#  tuples with fairly non-descriptive content.  This is modeled very much
#  after Lisp/Scheme - style pretty-printing of lists.  If you find it
#  useful, thank small children who sleep at night.

"""Support to pretty-print lists, tuples, & dictionaries recursively.

Very simple, but useful, especially in debugging data structures.

Classes
-------

PrettyPrinter()
    Handle pretty-printing operations onto a stream using a configured
    set of formatting parameters.

Functions
---------

pformat()
    Format a Python object into a pretty-printed representation.

pprint()
    Pretty-print a Python object to a stream [default is sys.stdout].

saferepr()
    Generate a 'standard' repr()-like value, but protect against recursive
    data structures.

"""

import collections as _collections
import dataclasses as _dataclasses
import re
import sys as _sys
import types as _types
from io import StringIO as _StringIO

__all__ = ["pprint","pformat","isreadable","isrecursive","saferepr",
           "PrettyPrinter", "pp"]


def pprint(object, stream=None, indent=1, width=80, depth=None, *,
           compact=False, sort_dicts=True, underscore_numbers=False):
    """Pretty-print a Python object to a stream [default is sys.stdout]."""
    printer = PrettyPrinter(
        stream=stream, indent=indent, width=width, depth=depth,
        compact=compact, sort_dicts=sort_dicts,
        underscore_numbers=underscore_numbers)
    printer.pprint(object)

def pformat(object, indent=1, width=80, depth=None, *,
            compact=False, sort_dicts=True, underscore_numbers=False):
    """Format a Python object into a pretty-printed representation."""
    return PrettyPrinter(indent=indent, width=width, depth=depth,
                         compact=compact, sort_dicts=sort_dicts,
                         underscore_numbers=underscore_numbers).pformat(object)

def pp(object, *args, sort_dicts=False, **kwargs):
    """Pretty-print a Python object"""
    pprint(object, *args, sort_dicts=sort_dicts, **kwargs)

def saferepr(object):
    """Version of repr() which can handle recursive data structures."""
    return PrettyPrinter()._safe_repr(object, {}, None, 0)[0]

def isreadable(object):
    """Determine if saferepr(object) is readable by eval()."""
    return PrettyPrinter()._safe_repr(object, {}, None, 0)[1]

def isrecursive(object):
    """Determine if object requires a recursive representation."""
    return PrettyPrinter()._safe_repr(object, {}, None, 0)[2]

class _safe_key:
    """Helper function for key functions when sorting unorderable objects.

    The wrapped-object will fallback to a Py2.x style comparison for
    unorderable types (sorting first comparing the type name and then by
    the obj ids).  Does not work recursively, so dict.items() must have
    _safe_key applied to both the key and the value.

    """

    __slots__ = ['obj']

    def __init__(self, obj):
        self.obj = obj

    def __lt__(self, other):
        try:
            return self.obj < other.obj
        except TypeError:
            return ((str(type(self.obj)), id(self.obj)) < \
                    (str(type(other.obj)), id(other.obj)))

def _safe_tuple(t):
    "Helper function for comparing 2-tuples"
    return _safe_key(t[0]), _safe_key(t[1])

class PrettyPrinter:
    def __init__(self, indent=1, width=80, depth=None, stream=None, *,
                 compact=False, sort_dicts=True, underscore_numbers=False):
        """Handle pretty printing operations onto a stream using a set of
        configured parameters.

        indent
            Number of spaces to indent for each level of nesting.

        width
            Attempted maximum number of columns in the output.

        depth
            The maximum depth to print out nested structures.

        stream
            The desired output stream.  If omitted (or false), the standard
            output stream available at construction will be used.

        compact
            If true, several items will be combined in one line.

        sort_dicts
            If true, dict keys are sorted.

        """
        indent = int(indent)
        width = int(width)
        if indent < 0:
            raise ValueError('indent must be >= 0')
        if depth is not None and depth <= 0:
            raise ValueError('depth must be > 0')
        if not width:
            raise ValueError('width must be != 0')
        self._depth = depth
        self._indent_per_level = indent
        self._width = width
        if stream is not None:
            self._stream = stream
        else:
            self._stream = _sys.stdout
        self._compact = bool(compact)
        self._sort_dicts = sort_dicts
        self._underscore_numbers = underscore_numbers

    def pprint(self, object):
        self._format(object, self._stream, 0, 0, {}, 0)
        self._stream.write("\n")

    def pformat(self, object):
        sio = _StringIO()
        self._format(object, sio, 0, 0, {}, 0)
        return sio.getvalue()

    def isrecursive(self, object):
        return self.format(object, {}, 0, 0)[2]

    def isreadable(self, object):
        s, readable, recursive = self.format(object, {}, 0, 0)
        return readable and not recursive

    def _format(self, object, stream, indent, allowance, context, level):
        objid = id(object)
        if objid in context:
            stream.write(_recursion(object))
            self._recursive = True
            self._readable = False
            return
        rep = self._repr(object, context, level)
        max_width = self._width - indent - allowance
        if len(rep) > max_width:
            p = self._dispatch.get(type(object).__repr__, None)
            if p is not None:
                context[objid] = 1
                p(self, object, stream, indent, allowance, context, level + 1)
                del context[objid]
                return
            elif (_dataclasses.is_dataclass(object) and
                  not isinstance(object, type) and
                  object.__dataclass_params__.repr and
                  # Check dataclass has generated repr method.
                  hasattr(object.__repr__, "__wrapped__") and
                  "__create_fn__" in object.__repr__.__wrapped__.__qualname__):
                context[objid] = 1
                self._pprint_dataclass(object, stream, indent, allowance, context, level + 1)
                del context[objid]
                return
        stream.write(rep)

    def _pprint_dataclass(self, object, stream, indent, allowance, context, level):
        cls_name = object.__class__.__name__
        indent += len(cls_name) + 1
        items = [(f.name, getattr(object, f.name)) for f in _dataclasses.fields(object) if f.repr]
        stream.write(cls_name + '(')
        self._format_namespace_items(items, stream, indent, allowance, context, level)
        stream.write(')')

    _dispatch = {}

    def _pprint_dict(self, object, stream, indent, allowance, context, level):
        write = stream.write
        write('{')
        if self._indent_per_level > 1:
            write((self._indent_per_level - 1) * ' ')
        length = len(object)
        if length:
            if self._sort_dicts:
                items = sorted(object.items(), key=_safe_tuple)
            else:
                items = object.items()
            self._format_dict_items(items, stream, indent, allowance + 1,
                                    context, level)
        write('}')

    _dispatch[dict.__repr__] = _pprint_dict

    def _pprint_ordered_dict(self, object, stream, indent, allowance, context, level):
        if not len(object):
            stream.write(repr(object))
            return
        cls = object.__class__
        stream.write(cls.__name__ + '(')
        self._format(list(object.items()), stream,
                     indent + len(cls.__name__) + 1, allowance + 1,
                     context, level)
        stream.write(')')

    _dispatch[_collections.OrderedDict.__repr__] = _pprint_ordered_dict

    def _pprint_list(self, object, stream, indent, allowance, context, level):
        stream.write('[')
        self._format_items(object, stream, indent, allowance + 1,
                           context, level)
        stream.write(']')

    _dispatch[list.__repr__] = _pprint_list

    def _pprint_tuple(self, object, stream, indent, allowance, context, level):
        stream.write('(')
        endchar = ',)' if len(object) == 1 else ')'
        self._format_items(object, stream, indent, allowance + len(endchar),
                           context, level)
        stream.write(endchar)

    _dispatch[tuple.__repr__] = _pprint_tuple

    def _pprint_set(self, object, stream, indent, allowance, context, level):
        if not len(object):
            stream.write(repr(object))
            return
        typ = object.__class__
        if typ is set:
            stream.write('{')
            endchar = '}'
        else:
            stream.write(typ.__name__ + '({')
            endchar = '})'
            indent += len(typ.__name__) + 1
        object = sorted(object, key=_safe_key)
        self._format_items(object, stream, indent, allowance + len(endchar),
                           context, level)
        stream.write(endchar)

    _dispatch[set.__repr__] = _pprint_set
    _dispatch[frozenset.__repr__] = _pprint_set

    def _pprint_str(self, object, stream, indent, allowance, context, level):
        write = stream.write
        if not len(object):
            write(repr(object))
            return
        chunks = []
        lines = object.splitlines(True)
        if level == 1:
            indent += 1
            allowance += 1
        max_width1 = max_width = self._width - indent
        for i, line in enumerate(lines):
            rep = repr(line)
            if i == len(lines) - 1:
                max_width1 -= allowance
            if len(rep) <= max_width1:
                chunks.append(rep)
            else:
                # A list of alternating (non-space, space) strings
                parts = re.findall(r'\S*\s*', line)
                assert parts
                assert not parts[-1]
                parts.pop()  # drop empty last part
                max_width2 = max_width
                current = ''
                for j, part in enumerate(parts):
                    candidate = current + part
                    if j == len(parts) - 1 and i == len(lines) - 1:
                        max_width2 -= allowance
                    if len(repr(candidate)) > max_width2:
                        if current:
                            chunks.append(repr(current))
                        current = part
                    else:
                        current = candidate
                if current:
                    chunks.append(repr(current))
        if len(chunks) == 1:
            write(rep)
            return
        if level == 1:
            write('(')
        for i, rep in enumerate(chunks):
            if i > 0:
                write('\n' + ' '*indent)
            write(rep)
        if level == 1:
            write(')')

    _dispatch[str.__repr__] = _pprint_str

    def _pprint_bytes(self, object, stream, indent, allowance, context, level):
        write = stream.write
        if len(object) <= 4:
            write(repr(object))
            return
        parens = level == 1
        if parens:
            indent += 1
            allowance += 1
            write('(')
        delim = ''
        for rep in _wrap_bytes_repr(object, self._width - indent, allowance):
            write(delim)
            write(rep)
            if not delim:
                delim = '\n' + ' '*indent
        if parens:
            write(')')

    _dispatch[bytes.__repr__] = _pprint_bytes

    def _pprint_bytearray(self, object, stream, indent, allowance, context, level):
        write = stream.write
        write('bytearray(')
        self._pprint_bytes(bytes(object), stream, indent + 10,
                           allowance + 1, context, level + 1)
        write(')')

    _dispatch[bytearray.__repr__] = _pprint_bytearray

    def _pprint_mappingproxy(self, object, stream, indent, allowance, context, level):
        stream.write('mappingproxy(')
        self._format(object.copy(), stream, indent + 13, allowance + 1,
                     context, level)
        stream.write(')')

    _dispatch[_types.MappingProxyType.__repr__] = _pprint_mappingproxy

    def _pprint_simplenamespace(self, object, stream, indent, allowance, context, level):
        if type(object) is _types.SimpleNamespace:
            # The SimpleNamespace repr is "namespace" instead of the class
            # name, so we do the same here. For subclasses; use the class name.
            cls_name = 'namespace'
        else:
            cls_name = object.__class__.__name__
        indent += len(cls_name) + 1
        items = object.__dict__.items()
        stream.write(cls_name + '(')
        self._format_namespace_items(items, stream, indent, allowance, context, level)
        stream.write(')')

    _dispatch[_types.SimpleNamespace.__repr__] = _pprint_simplenamespace

    def _format_dict_items(self, items, stream, indent, allowance, context,
                           level):
        write = stream.write
        indent += self._indent_per_level
        delimnl = ',\n' + ' ' * indent
        last_index = len(items) - 1
        for i, (key, ent) in enumerate(items):
            last = i == last_index
            rep = self._repr(key, context, level)
            write(rep)
            write(': ')
            self._format(ent, stream, indent + len(rep) + 2,
                         allowance if last else 1,
                         context, level)
            if not last:
                write(delimnl)

    def _format_namespace_items(self, items, stream, indent, allowance, context, level):
        write = stream.write
        delimnl = ',\n' + ' ' * indent
        last_index = len(items) - 1
        for i, (key, ent) in enumerate(items):
            last = i == last_index
            write(key)
            write('=')
            if id(ent) in context:
                # Special-case representation of recursion to match standard
                # recursive dataclass repr.
                write("...")
            else:
                self._format(ent, stream, indent + len(key) + 1,
                             allowance if last else 1,
                             context, level)
            if not last:
                write(delimnl)

    def _format_items(self, items, stream, indent, allowance, context, level):
        write = stream.write
        indent += self._indent_per_level
        if self._indent_per_level > 1:
            write((self._indent_per_level - 1) * ' ')
        delimnl = ',\n' + ' ' * indent
        delim = ''
        width = max_width = self._width - indent + 1
        it = iter(items)
        try:
            next_ent = next(it)
        except StopIteration:
            return
        last = False
        while not last:
            ent = next_ent
            try:
                next_ent = next(it)
            except StopIteration:
                last = True
                max_width -= allowance
                width -= allowance
            if self._compact:
                rep = self._repr(ent, context, level)
                w = len(rep) + 2
                if width < w:
                    width = max_width
                    if delim:
                        delim = delimnl
                if width >= w:
                    width -= w
                    write(delim)
                    delim = ', '
                    write(rep)
                    continue
            write(delim)
            delim = delimnl
            self._format(ent, stream, indent,
                         allowance if last else 1,
                         context, level)

    def _repr(self, object, context, level):
        repr, readable, recursive = self.format(object, context.copy(),
                                                self._depth, level)
        if not readable:
            self._readable = False
        if recursive:
            self._recursive = True
        return repr

    def format(self, object, context, maxlevels, level):
        """Format object for a specific context, returning a string
        and flags indicating whether the representation is 'readable'
        and whether the object represents a recursive construct.
        """
        return self._safe_repr(object, context, maxlevels, level)

    def _pprint_default_dict(self, object, stream, indent, allowance, context, level):
        if not len(object):
            stream.write(repr(object))
            return
        rdf = self._repr(object.default_factory, context, level)
        cls = object.__class__
        indent += len(cls.__name__) + 1
        stream.write('%s(%s,\n%s' % (cls.__name__, rdf, ' ' * indent))
        self._pprint_dict(object, stream, indent, allowance + 1, context, level)
        stream.write(')')

    _dispatch[_collections.defaultdict.__repr__] = _pprint_default_dict

    def _pprint_counter(self, object, stream, indent, allowance, context, level):
        if not len(object):
            stream.write(repr(object))
            return
        cls = object.__class__
        stream.write(cls.__name__ + '({')
        if self._indent_per_level > 1:
            stream.write((self._indent_per_level - 1) * ' ')
        items = object.most_common()
        self._format_dict_items(items, stream,
                                indent + len(cls.__name__) + 1, allowance + 2,
                                context, level)
        stream.write('})')

    _dispatch[_collections.Counter.__repr__] = _pprint_counter

    def _pprint_chain_map(self, object, stream, indent, allowance, context, level):
        if not len(object.maps):
            stream.write(repr(object))
            return
        cls = object.__class__
        stream.write(cls.__name__ + '(')
        indent += len(cls.__name__) + 1
        for i, m in enumerate(object.maps):
            if i == len(object.maps) - 1:
                self._format(m, stream, indent, allowance + 1, context, level)
                stream.write(')')
            else:
                self._format(m, stream, indent, 1, context, level)
                stream.write(',\n' + ' ' * indent)

    _dispatch[_collections.ChainMap.__repr__] = _pprint_chain_map

    def _pprint_deque(self, object, stream, indent, allowance, context, level):
        if not len(object):
            stream.write(repr(object))
            return
        cls = object.__class__
        stream.write(cls.__name__ + '(')
        indent += len(cls.__name__) + 1
        stream.write('[')
        if object.maxlen is None:
            self._format_items(object, stream, indent, allowance + 2,
                               context, level)
            stream.write('])')
        else:
            self._format_items(object, stream, indent, 2,
                               context, level)
            rml = self._repr(object.maxlen, context, level)
            stream.write('],\n%smaxlen=%s)' % (' ' * indent, rml))

    _dispatch[_collections.deque.__repr__] = _pprint_deque

    def _pprint_user_dict(self, object, stream, indent, allowance, context, level):
        self._format(object.data, stream, indent, allowance, context, level - 1)

    _dispatch[_collections.UserDict.__repr__] = _pprint_user_dict

    def _pprint_user_list(self, object, stream, indent, allowance, context, level):
        self._format(object.data, stream, indent, allowance, context, level - 1)

    _dispatch[_collections.UserList.__repr__] = _pprint_user_list

    def _pprint_user_string(self, object, stream, indent, allowance, context, level):
        self._format(object.data, stream, indent, allowance, context, level - 1)

    _dispatch[_collections.UserString.__repr__] = _pprint_user_string

    def _safe_repr(self, object, context, maxlevels, level):
        # Return triple (repr_string, isreadable, isrecursive).
        typ = type(object)
        if typ in _builtin_scalars:
            return repr(object), True, False

        r = getattr(typ, "__repr__", None)

        if issubclass(typ, int) and r is int.__repr__:
            if self._underscore_numbers:
                return f"{object:_d}", True, False
            else:
                return repr(object), True, False

        if issubclass(typ, dict) and r is dict.__repr__:
            if not object:
                return "{}", True, False
            objid = id(object)
            if maxlevels and level >= maxlevels:
                return "{...}", False, objid in context
            if objid in context:
                return _recursion(object), False, True
            context[objid] = 1
            readable = True
            recursive = False
            components = []
            append = components.append
            level += 1
            if self._sort_dicts:
                items = sorted(object.items(), key=_safe_tuple)
            else:
                items = object.items()
            for k, v in items:
                krepr, kreadable, krecur = self.format(
                    k, context, maxlevels, level)
                vrepr, vreadable, vrecur = self.format(
                    v, context, maxlevels, level)
                append("%s: %s" % (krepr, vrepr))
                readable = readable and kreadable and vreadable
                if krecur or vrecur:
                    recursive = True
            del context[objid]
            return "{%s}" % ", ".join(components), readable, recursive

        if (issubclass(typ, list) and r is list.__repr__) or \
           (issubclass(typ, tuple) and r is tuple.__repr__):
            if issubclass(typ, list):
                if not object:
                    return "[]", True, False
                format = "[%s]"
            elif len(object) == 1:
                format = "(%s,)"
            else:
                if not object:
                    return "()", True, False
                format = "(%s)"
            objid = id(object)
            if maxlevels and level >= maxlevels:
                return format % "...", False, objid in context
            if objid in context:
                return _recursion(object), False, True
            context[objid] = 1
            readable = True
            recursive = False
            components = []
            append = components.append
            level += 1
            for o in object:
                orepr, oreadable, orecur = self.format(
                    o, context, maxlevels, level)
                append(orepr)
                if not oreadable:
                    readable = False
                if orecur:
                    recursive = True
            del context[objid]
            return format % ", ".join(components), readable, recursive

        rep = repr(object)
        return rep, (rep and not rep.startswith('<')), False

_builtin_scalars = frozenset({str, bytes, bytearray, float, complex,
                              bool, type(None)})

def _recursion(object):
    return ("<Recursion on %s with id=%s>"
            % (type(object).__name__, id(object)))


def _perfcheck(object=None):
    import time
    if object is None:
        object = [("string", (1, 2), [3, 4], {5: 6, 7: 8})] * 100000
    p = PrettyPrinter()
    t1 = time.perf_counter()
    p._safe_repr(object, {}, None, 0, True)
    t2 = time.perf_counter()
    p.pformat(object)
    t3 = time.perf_counter()
    print("_safe_repr:", t2 - t1)
    print("pformat:", t3 - t2)

def _wrap_bytes_repr(object, width, allowance):
    current = b''
    last = len(object) // 4 * 4
    for i in range(0, len(object), 4):
        part = object[i: i+4]
        candidate = current + part
        if i == last:
            width -= allowance
        if len(repr(candidate)) > width:
            if current:
                yield repr(current)
            current = part
        else:
            current = candidate
    if current:
        yield repr(current)

if __name__ == "__main__":
    _perfcheck()
